<template>
  <div class="timer-set">
    <!-- 定时管理 -->
    <card-group>
      <template v-slot:content>
        <div class="timer-manage">
          <!-- 定时列表 -->
          <div v-if="timerTasks.length" class="timer-manage-list">
            <!-- 标题 -->
            <div class="timer-manage-list-header">
              <gree-check v-if="delType" v-model="timerSelectAllFlag" :label="$language('timer.allSelect')" />
              <span v-if="!delType && timerTasks.length <= 3" v-text="$language('timer.management')" @click="delType = true" />
              <span v-if="timerTasks.length > 3" @click="checkAllTimeTask">{{ $language('timer.all_timer') }}<gree-icon name="arrow-right" size="sm"/></span>
            </div>
            <!-- 列表内容 -->
            <div
              class="timer-manage-list-content"
              v-for="(timerTask, timerTaskIndex) in timerTasks.slice(0, 3)"
              :key="timerTaskIndex"
              @click="selectOne(timerTaskIndex)"
            >
              <!-- 左边内容 -->
              <div class="timer-manage-list-content-left">
                <p class="timer-manage-list-content-left-header">
                  <!-- 时间 -->
                  <span v-text="parseTime(timerTask.timer.Etime, timerTask.timer.timeZone)" />
                  <!-- 动作 -->
                  <span v-html="`&nbsp; ${getPowType(timerTask.task.cmd) ? $language('timer.turn_off') : $language('timer.turn_on')}`" />
                </p>
                <p class="timer-manage-list-content-left-footer" v-text="parseDate(timerTask.timer.weeks) || ''" />
              </div>
              <!-- 右边内容 -->
              <div class="timer-manage-list-content-right" @click.stop>
                <gree-check v-if="delType" class="card-item-right-check" v-model="timerSelcetList[timerTaskIndex]" />
                <gree-switch
                  v-else
                  class="card-item-right-switch"
                  inactive-color="#c5c5c5"
                  :size="switchSize"
                  v-model="listSwitchType[timerTaskIndex]"
                  @change="changeTask(timerTaskIndex)"
                />
              </div>
            </div>
            <!-- 删除按钮 -->
            <div v-if="delType" class="timer-manage-list-del">
              <!-- 取消 -->
              <div
                class="timer-manage-list-del-cancel"
                v-text="$language('timer.btn_cancel')"
                @click="
                  delType = false;
                  timerSelectAllFlag = false;
                "
              />
              <!-- 确定 -->
              <div class="timer-manage-list-del-confirm" :class="{ disable: delAble }" v-text="$language('timer.btn_delet')" @click="onDelete" />
            </div>
            <!-- 添加按钮 -->
            <div class="timer-manage-list-add" v-else>
              <!-- 点击热区 -->
              <div
                class="timer-manage-list-add-box"
                @click="
                  setPickerPropData();
                  showTimerSetPopup(true);
                "
              >
                <!-- icon -->
                <gree-icon name="add" size="26" />
                <!-- 文本 -->
                <span class="timer-manage-list-add-box-text" v-text="$language('timer.add')" />
              </div>
            </div>
          </div>
          <!-- 大大的添加按钮 -->
          <div
            v-else
            class="timer-manage-blank"
            @click="
              setPickerPropData();
              showTimerSetPopup(true);
            "
          >
            <div class="timer-manage-blank-add">
              <div class="timer-manage-blank-add-icon" />
              <span class="timer-manage-blank-add-text"
                >{{ $language('timer.null_tips1') }}<a style="text-decoration: underline">{{ $language('timer.null_tips2') }}</a></span
              >
            </div>
          </div>
        </div>
      </template>
    </card-group>
    <!-- 弹框设置 -->
    <TimerPicker @confirm="getPickerData" :timer-task-time="timerTaskTime" :timer-task-weeks="timerTaskWeeks" :timer-i-d="timerID" />
  </div>
</template>

<script>
import { Switch, Check, Dialog, Toast, Icon } from 'gree-ui';
import { setCloudTimer, modifyCloudTimer, deleteCloudTimer, getCloudTimerByMac, deleteLocalTimer, getAllTimerList } from '@PluginInterface';
import { mapState, mapMutations, mapGetters } from 'vuex';
import { analysisTimer } from '@/utils/encryptionFun';
import CardGroup from '@/components/CardGroup/index';
import TimerPicker from '@/components/Popup/TimerPicker';

export default {
  name: 'timerSetGroup',
  components: {
    [CardGroup.name]: CardGroup,
    [TimerPicker.name]: TimerPicker,
    [Switch.name]: Switch,
    [Check.name]: Check,
    [Icon.name]: Icon
  },
  data() {
    return {
      timerSelcetList: [],
      timerSelectAllFlag: false,
      timerTasks: [],
      listSwitchType: [],
      delType: false, // 删除状态
      timerTaskTime: {},
      timerTaskWeeks: [],
      timerID: 0,
      delAble: false,
      isShowLoading: false
    };
  },
  activated() {
    // 查一包云定时
    this.localGetCloudTimer();
  },
  mounted() {
    //本地定时更新位云定时（仅内销家用），出口项目或商用项目打包时记得屏蔽（产品抽风，先屏蔽）
    // this.changeTimerTaskToCloud();
  },
  computed: {
    ...mapState('control', {
      mac: state => state.mac,
      mainMac: state => state.mainMac
    }),
    ...mapGetters(['cardFuncDefine', 'modTextKey', 'modCurrentStatusName', 'spModCurrentStatusName']),
    isAllSelect() {
      return this.timerTasks.every((v, i) => {
        return this.timerSelcetList[i];
      });
    },
    isAllBlur() {
      return !this.timerTasks.some((v, i) => {
        return this.timerSelcetList[i];
      });
    },
    switchSize() {
      return Math.floor((window['lib'].flexible.rem / 108) * 88);
    }
  },
  watch: {
    'timerTasks.length': {
      handler() {
        // 更新高度
        this.updateBoxHeight();
        this.selectAll();
      },
      immediate: true
    },
    timerSelectAllFlag(newVal) {
      newVal !== 0 && this.selectAll(newVal);
    },
    isAllSelect(newVal) {
      if (newVal) {
        this.timerSelectAllFlag = true;
      } else {
        this.timerSelectAllFlag = 0;
      }
    },
    timerSelcetList() {
      this.timerSelcetList.includes(true) ? (this.delAble = false) : (this.delAble = true);
    },
    isAllBlur(newVal) {
      newVal && (this.timerSelectAllFlag = false);
    }
  },
  methods: {
    ...mapMutations('control', {
      setDataObject: 'SET_DATA_OBJECT',
      setState: 'CONTROL_SET_STATE',
      SetPopup: 'SET_POPUP'
    }),
    changeTimerTaskToCloud() {
      getAllTimerList('', this.mac).then(_res => {
        try {
          const checkTaskList = JSON.parse(_res);
          // console.log('旧定时列表:', checkTaskList);
          if (!checkTaskList.length || this.timerTasks.length >= checkTaskList.length || this.timerTasks.length >= 10) return;
          this.$toast('正在更新定时，请勿操作定时功能！');
          for (let i = 0; i < checkTaskList.length; i++) {
            // console.log(`第${i + 1}条定时`, checkTaskList[i]);
            const { enable, hr, min, week, cmd } = checkTaskList[i];
            // console.log(`第${i + 1}条定时中需要用到的参数`, enable, hr, min, week, cmd[0].p[0]);
            const weeks = this.findall(week, 1);
            // console.log(`第${i + 1}条定时选了周几：`, weeks.map(String));
            const { mac, mainMac } = this;
            const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac
            const hour = (((hr + (24 - 8)) % 24) + '').padStart(2, '0');
            const minus = (min + '').padStart(2, '0');
            const Etime = `${hour}:${minus}:00`;
            const Pow = Number(cmd[0].p[0]);
            const p = [Pow, 0, 0, 0, 0];
            const opt = ['Pow', 'Health', 'SlpMod', 'SwhSlp', 'StHt'];
            const timerDic = {
              timeType: weeks.length ? 2 : 1,
              weeks: weeks.map(String),
              Etime,
              timeZone: 8,
              status: Number(!enable)
            };
            const taskDic = {
              mac: sendMac,
              subId: mainMac.length ? mac : '', //多联机subId需要传入子mac
              cmd: JSON.stringify({ t: 'cmd', sub: mac, opt, p }),
              remark: 'i am lazy, no remark',
              dat: p.map(String)
            };
            setCloudTimer(timerDic, taskDic).then(_res => {
              const res = JSON.parse(_res);
              console.log('设置定时后：', res);
              this.localGetCloudTimer();
            });
          }
          const delTaskId = [];
          for (let i = 0; i < checkTaskList.length; i++) {
            const { id } = checkTaskList[i];
            delTaskId.push(id);
          }
          console.log('delTaskId:', delTaskId.map(String));
          deleteLocalTimer(this.mac, delTaskId.map(String)).then(_res => {
            const res = JSON.parse(_res);
            res.r === 200 && this.$toast('更新成功');
            res.r === 404 && this.$toast('更新失败');
            console.log('删除本地定时：', res);
          });
        } catch (e) {
          console.log('err', e);
          this.$toast('更新失败');
        }
      });
    },
    //返回x在数组中的所有索引
    findall(arr, x) {
      var results = [],
        len = arr.length,
        pos = 0;
      while (pos < len) {
        pos = arr.indexOf(x, pos);
        if (pos === -1) {
          //未找到就退出循环完成搜索
          break;
        }
        results.push(pos); //找到就存储索引
        pos += 1; //并从下个位置开始搜索
      }
      results.forEach(() => {
        !results[0] && (results[0] = 7); //模块定时的周日是0，换成云定时需要强行置为7
      });
      results.sort(function(firstValue, secondValue) {
        return firstValue - secondValue;
      });
      return results;
    },
    localGetCloudTimer() {
      const { mac, mainMac } = this;
      const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac
      return getCloudTimerByMac(sendMac).then(res => {
        const result = JSON.parse(res);
        if (result.r === 200) {
          this.timerTasks = analysisTimer(result.timerTasks).reverse();
          // 更新开关状态
          const hasTaskOn = this.updateSwitchType();
          this.setDataObject({ TimerSet: hasTaskOn });
          // 更新定时小图标文案
          this.getCloseTimer();
          this.isShowLoading = false;
        }
      });
    },
    // 更新开关状态
    updateSwitchType() {
      let hasTaskOn = false;
      this.listSwitchType = this.timerTasks
        ? this.timerTasks.map(timerTask => {
            const type = Boolean(timerTask.timer.status);
            hasTaskOn = hasTaskOn || type;
            return type;
          })
        : [];
      return hasTaskOn;
    },
    // 更新高度
    updateBoxHeight() {
      this.$emit('updateHeight');
    },
    // 获取开关状态 1关 0开
    getPowType(cmd) {
      const { opt, p } = cmd;
      const powIndex = opt.findIndex(v => v === 'Pow');
      const powType = powIndex === -1 ? 1 : Number(!p[powIndex]);
      return powType;
    },
    // 时间解析
    parseTime(stringTime, timeZone) {
      if (!stringTime) return;
      const timeList = stringTime.split(':');
      let hour = Number(timeList[0]) + timeZone;
      if (hour >= 24) hour -= 24;
      if (hour < 10) hour = `0${hour}`;
      return `${hour}:${timeList[1]}  `;
    },
    parseTimeToMap(timer, task) {
      const { Etime, weeks, timeZone, timerID } = timer;
      const targetTime = this.parseTime(Etime, timeZone);
      const type = this.getPowType(task.cmd);
      const [hour, minus] = targetTime.split(':').map(Number);
      return {
        hour,
        minus,
        type,
        weeks,
        timerID
      };
    },
    // 解析日期
    parseDate(list) {
      if (!list) return;
      if (list.length === 7) return this.$language('timer.every');
      if (list.length === 0 || list[0] === '') return this.$language('timer.once');
      const valueObj = this.$language('timer.Mon', 'timer.Tue', 'timer.Wed', 'timer.Thu', 'timer.Fir', 'timer.Sat', 'timer.Sun');
      let returnValue = '';
      list.forEach(element => {
        if (element === '') return;
        returnValue += `${valueObj[Number(element) - 1]} `;
      });
      return returnValue.substring(0, returnValue.length - 1);
    },
    async showTimerSetPopup(isCheck = false) {
      if (isCheck) {
        await this.localGetCloudTimer();
        if (this.timerTasks.length >= 10) {
          this.$toast(`${this.$language('timer.toast')}`);
          return;
        }
      }
      this.SetPopup({ timerPicker: true });
    },
    changeTask(index) {
      this.showLoading();
      const { task, timer } = this.timerTasks[index];
      const { mac, mainMac } = this;
      const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac

      const timerDic = {
        timeType: timer.timeType,
        weeks: timer.weeks,
        Etime: timer.Etime,
        timeZone: timer.timeZone,
        status: Number(!timer.status)
      };
      const taskDic = {
        mac: sendMac,
        subId: mainMac.length ? mac : '',
        cmd: JSON.stringify(task.cmd),
        remark: task.remark,
        dat: task.dat
      };
      modifyCloudTimer(timerDic, taskDic, timer.timerID).then(() => {
        // 保存成功主动查一次
        this.localGetCloudTimer();
        this.hideLoading();
      });
    },
    //删除定时弹框
    onDelete() {
      Dialog.confirm({
        message: this.$language('timer.Dialog_content'),
        confirmButtonText: this.$language('timer.Dialog_confirmText'),
        cancelButtonText: this.$language('timer.Dialog_cancelText'),
        confirmButtonColor: '#ff0202'
      })
        .then(() => {
          this.delTimers();
        })
        .catch(() => {
          return;
        });
    },
    delTimers() {
      this.delType = false;
      this.timerSelectAllFlag = false;
      const timerIDs = [];
      this.timerSelcetList.forEach((value, index) => {
        try {
          value && timerIDs.push(this.timerTasks[index].timer.timerID);
        } catch (e) {
          e;
        }
      });
      deleteCloudTimer(timerIDs).then(() => {
        this.$toast(`${this.$language('timer.Delete_success')}`);
        this.localGetCloudTimer();
      });
    },
    setPickerPropData(index = -1) {
      if (index === -1) {
        //新增定时
        const now = new Date();
        this.timerTaskTime = { hour: now.getHours(), minus: now.getMinutes(), type: 0 };
        this.timerTaskWeeks = [];
        this.timerID = 0;
      } else {
        //修改定时
        const { timer, task } = this.timerTasks[index];
        const { hour, minus, type, weeks, timerID } = this.parseTimeToMap(timer, task);

        this.timerTaskTime = {
          hour,
          minus,
          type
        };
        this.timerTaskWeeks = weeks;
        this.timerID = timerID;
      }
    },
    selectOne(index) {
      if (this.delType) {
        this.$set(this.timerSelcetList, index, !this.timerSelcetList[index]);
      } else {
        // 设置picker数据
        this.setPickerPropData(index);
        // 显示picker弹窗
        this.showTimerSetPopup();
      }
    },
    selectAll(type) {
      const setTimerAll = type => {
        this.timerSelcetList = this.timerTasks.map(() => type);
      };
      if (type === undefined) {
        this.isAllSelect && setTimerAll(true);
        this.isAllBlur && setTimerAll(false);
      } else {
        setTimerAll(type);
      }
    },
    async getPickerData(val) {
      await this.localGetCloudTimer();
      const { mac, mainMac } = this;
      const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac

      const [pickerData, weekList, timerID] = val;
      const hour = (((Number(pickerData[0]) + (24 - 8)) % 24) + '').padStart(2, '0');
      const minus = (pickerData[1] + '').padStart(2, '0');
      const Etime = `${hour}:${minus}:00`;
      const Pow = pickerData[2].includes(this.$t('timer.turn_on')) ? 1 : 0;
      const p = [Pow, 0, 0, 0, 0];
      const opt = ['Pow', 'Health', 'SlpMod', 'SwhSlp', 'StHt'];

      const weeks = [];
      weekList.forEach(week => {
        week.select && weeks.push(week.index);
      });
      const timerDic = {
        timeType: weeks.length ? 2 : 1,
        weeks,
        Etime,
        timeZone: 8,
        status: 1
      };
      const taskDic = {
        mac: sendMac,
        subId: mainMac.length ? mac : '', //多联机subId需要传入子mac
        cmd: JSON.stringify({ t: 'cmd', sub: mac, opt, p }),
        remark: 'i am lazy, no remark',
        dat: p.map(String)
      };

      if (timerID && this.timerTasks.find(timerTask => timerTask.timer.timerID === timerID)) {
        modifyCloudTimer(timerDic, taskDic, timerID).then(() => {
          this.$toast(`${this.$language('timer.toast1')}`);
          // 保存成功主动查一次
          this.localGetCloudTimer();
        });
      } else {
        if (this.timerTasks.length >= 10) {
          this.$toast(`${this.$language('timer.toast')}`);
        } else {
          console.log('setCloudTimer--------------参数：', JSON.stringify(timerDic), JSON.stringify(taskDic));
          setCloudTimer(timerDic, taskDic).then(() => {
            this.$toast(`${this.$language('timer.toast1')}`);
            // 保存成功主动查一次
            this.localGetCloudTimer();
          });
        }
      }
    },
    // 最近的一个定时
    getCloseTimer() {
      const timerTaskList = this.timerTasks.filter(timerTask => timerTask.timer.status);
      const date = new Date();
      const timerList = [];
      const targetTimer = timerTaskList
        .map(timerTask => {
          let diffTime;
          const { timer, task } = timerTask;
          const { hour, minus, type, weeks, timerID } = this.parseTimeToMap(timer, task);
          timerList.push({ hour, minus, type, timerID });
          if (weeks.length && weeks[0] !== '') {
            // 轮询选择的日期，返回最近的结果
            diffTime = weeks
              .map(day => {
                return date.dateDiff({
                  hour,
                  minus,
                  day: day % 7
                });
              })
              .sort((a, b) => a.time - b.time)[0];
          } else {
            // 返回当前时间与选择时间的差值
            diffTime = date.dateDiff({
              hour,
              minus
            });
          }
          return { timerID, ...diffTime };
        })
        .sort((a, b) => a.time - b.time)[0];
      const result = timerList && timerList.find(timer => timer.timerID === targetTimer.timerID);
      if (!result) {
        this.setState({ closeTimer: '' });
      } else {
        const timeString = `${this.padZero(result.hour)}:${this.padZero(result.minus)}`;
        this.setState({ closeTimer: `${timeString}${result.type ? this.$language('miniIcon.timerOff') : this.$language('miniIcon.timerOn')}` });
      }
    },
    //开关颜色
    // switchColor() {
    //   let result = '';
    //   const modArr = ['status_1', 'status_5', 'status_6', 'status_7'];
    //   if (modArr.includes(this.modCurrentStatusName)) {
    //     result = '#ffac83';
    //   } else {
    //     result = '#71ACF1';
    //   }
    //   return result;
    // },
    //数字补零
    padZero(num, targetLength = 2) {
      let str = num + '';
      while (str.length < targetLength) {
        str = '0' + str;
      }
      return str;
    },
    checkAllTimeTask() {
      this.$router.push('TimeTaskSet');
    },
    showLoading() {
      Toast.loading('');
    },
    hideLoading() {
      !this.isShowLoading && Toast.clear();
    }
  }
};
</script>

<style lang="scss">
.timer-manage {
  width: 100%;
  display: flex;
  justify-content: center;
  align-items: flex-start;
  &-list {
    height: 100%;
    width: 100%;
    &-header {
      $headerHeight: 116px;
      height: $headerHeight;
      line-height: $headerHeight;
      width: 100%;
      display: flex;
      font-size: 44px;
      justify-content: flex-end;
      > label {
        width: auto;
        margin-right: 58px;
        .gree-check-icon {
          position: relative;
          color: #c5cad5;
          .gree-icon {
            color: #71acf1;
          }
        }
      }
      > span {
        margin-right: 60px;
        color: #404657;
        .gree-icon {
          padding-left: 20px;
          color: #404657;
        }
      }
    }
    &-content {
      height: 150px;
      background-image: linear-gradient(0deg, rgba(207, 204, 204, 0.2), rgba(207, 204, 204, 0.2));
      background-size: 100% 1px;
      background-repeat: no-repeat;
      background-position: top;
      display: flex;
      justify-content: space-between;
      align-items: center;
      font-size: 46px;
      &-left {
        padding-left: 52px;
        color: #404657;
        &-header {
          height: 54px;
        }
        &-footer {
          height: 32px;
          font-size: 36px;
          color: #989898;
        }
      }
      &-right {
        padding-right: 58px;
        .card-item-right-check {
          .gree-check-icon {
            color: #c5cad5;
            .gree-icon {
              color: #71acf1;
            }
          }
        }
        .gree-switch {
          top: 10px;
        }
      }
    }
    &-del {
      $delHeight: 168px;
      width: 100%;
      height: $delHeight;
      background-image: linear-gradient(0deg, rgba(152, 152, 152, 0.2), rgba(152, 152, 152, 0.15), transparent);
      background-size: 100% 1px;
      background-repeat: no-repeat;
      background-position: top;
      font-size: 44px;
      color: #ff0202;
      display: flex;
      justify-content: space-between;
      &-cancel {
        color: #404657;
        // border-right: 1px solid #f0f0f0;
        background-image: linear-gradient(0deg, rgba(152, 152, 152, 0.2), rgba(152, 152, 152, 0.15), transparent);
        background-size: 1px 100%;
        background-repeat: no-repeat;
        background-position: right;
      }
      &-confirm {
        &.disable {
          color: gray;
          opacity: 0.3;
          pointer-events: none;
        }
      }
      div {
        width: 50%;
        height: 100%;
        text-align: center;
        line-height: $delHeight;
      }
    }
    &-add {
      width: 100%;
      height: 168px;
      background-image: linear-gradient(0deg, rgba(207, 204, 204, 0.2), rgba(207, 204, 204, 0.2));
      background-size: 100% 1px;
      background-repeat: no-repeat;
      background-position: top;
      display: flex;
      align-items: center;
      justify-content: center;
      &-box {
        width: auto;
        height: 80%;
        padding: 0 20px;
        display: flex;
        align-items: center;
        justify-content: center;
        &-text {
          font-size: 44px;
          color: #404657;
          padding-top: 5px;
          padding-left: 20px;
        }
      }
    }
    &-check {
      width: 100%;
      height: 168px;
      background-image: linear-gradient(0deg, rgba(207, 204, 204, 0.2), rgba(207, 204, 204, 0.2));
      background-size: 100% 1px;
      background-repeat: no-repeat;
      background-position: top;
      display: flex;
      align-items: center;
      justify-content: center;
      &-box {
        width: auto;
        height: 80%;
        padding: 0 20px;
        display: flex;
        align-items: center;
        justify-content: center;
        &-text {
          font-size: 44px;
          color: #404657;
        }
      }
    }
  }
  &-blank {
    height: 600px;
    width: 100%;
    display: flex;
    justify-content: center;
    align-items: center;
    &-add {
      text-align: center;
      &-icon {
        $iconHeight: 191px;
        $iconWidth: 223px;
        height: $iconHeight;
        width: $iconWidth;
        background-image: url(~@assets/img/functionBtn/timer/notiming.png);
        background-position: center;
        background-repeat: no-repeat;
        background-size: cover;
        margin: auto;
        // transform: translate(30%, -20px);
        // line-height: $iconHeight;
        // border-radius: 50%;
        // border: 1px solid #404657;
        // font-family: 'Roboto-Light';
        // text-align: center;
        // margin-bottom: 32px;
        // &::after {
        //   color: #404657;
        //   font-size: 120px;
        //   content: '+';
        // }
      }
      &-text {
        font-size: 40px;
        color: #404657;
        a {
          color: #71acf1;
        }
      }
    }
  }
}
</style>
